@@ -33,6 +33,9 @@
 		},
 		// update quota
 		updateStorageQuotas: function() {
+			Files._updateStorageQuotasThrottled();
+		},
+		_updateStorageQuotas: function() {
 			var state = Files.updateStorageQuotas;
 			state.call = $.getJSON(OC.filePath('files','ajax','getstoragestats.php'),function(response) {
 				Files.updateQuota(response);
@@ -69,6 +72,7 @@
 				$('#free_space').val(response.data.freeSpace);
 				$('#upload.button').attr('data-original-title', response.data.maxHumanFilesize);
 				$('#usedSpacePercent').val(response.data.usedSpacePercent);
+				$('#usedSpacePercent').data('mount-type', response.data.mountType);
 				$('#owner').val(response.data.owner);
 				$('#ownerDisplayName').val(response.data.ownerDisplayName);
 				Files.displayStorageWarnings();
@@ -137,7 +141,7 @@
 				throw t('files', 'File name cannot be empty.');
 			} else if (trimmedName.indexOf('/') !== -1) {
 				throw t('files', '"/" is not allowed inside a file name.');
-			} else if (OC.fileIsBlacklisted(trimmedName)) {
+			} else if (!!(trimmedName.match(OC.config.blacklist_files_regex))) {
 				throw t('files', '"{name}" is not an allowed filetype', {name: name});
 			}
 
@@ -150,22 +154,31 @@
 
 			var usedSpacePercent = $('#usedSpacePercent').val(),
 				owner = $('#owner').val(),
-				ownerDisplayName = $('#ownerDisplayName').val();
+				ownerDisplayName = $('#ownerDisplayName').val(),
+				mountType = $('#usedSpacePercent').data('mount-type');
 			if (usedSpacePercent > 98) {
-				if (owner !== oc_current_user) {
+				if (owner !== OC.getCurrentUser().uid) {
 					OC.Notification.show(t('files', 'Storage of {owner} is full, files can not be updated or synced anymore!',
 						{owner: ownerDisplayName}), {type: 'error'}
 					);
-					return;
+				} else if (mountType === 'group') {
+					OC.Notification.show(t('files',
+						'This group folder is full, files can not be updated or synced anymore!'),
+						{type: 'error'}
+					);
+				} else if (mountType === 'external') {
+					OC.Notification.show(t('files',
+						'This external storage is full, files can not be updated or synced anymore!'),
+						{type : 'error'}
+					);
+				} else {
+					OC.Notification.show(t('files',
+						'Your storage is full, files can not be updated or synced anymore!'),
+						{type: 'error'}
+					);
 				}
-				OC.Notification.show(t('files',
-					'Your storage is full, files can not be updated or synced anymore!'),
-					{type : 'error'}
-				);
-				return;
-			}
-			if (usedSpacePercent > 90) {
-				if (owner !== oc_current_user) {
+			} else if (usedSpacePercent > 90) {
+				if (owner !== OC.getCurrentUser().uid) {
 					OC.Notification.show(t('files', 'Storage of {owner} is almost full ({usedSpacePercent}%)',
 						{
 							usedSpacePercent: usedSpacePercent,
@@ -175,12 +188,24 @@
 							type: 'error'
 						}
 					);
-					return;
+				} else if (mountType === 'group') {
+					OC.Notification.show(t('files',
+						'This group folder is almost full ({usedSpacePercent}%)',
+						{usedSpacePercent: usedSpacePercent}),
+						{type : 'error'}
+					);
+				} else if (mountType === 'external') {
+					OC.Notification.show(t('files',
+						'This external storage is almost full ({usedSpacePercent}%)',
+						{usedSpacePercent: usedSpacePercent}),
+						{type : 'error'}
+					);
+				} else {
+					OC.Notification.show(t('files', 'Your storage is almost full ({usedSpacePercent}%)',
+						{usedSpacePercent: usedSpacePercent}),
+						{type : 'error'}
+					);
 				}
-				OC.Notification.show(t('files', 'Your storage is almost full ({usedSpacePercent}%)',
-					{usedSpacePercent: usedSpacePercent}),
-					{type : 'error'}
-				);
 			}
 		},
 
@@ -286,7 +311,7 @@
 			setTimeout(Files.displayStorageWarnings, 100);
 
 			// only possible at the moment if user is logged in or the files app is loaded
-			if (OC.currentUser && OCA.Files.App) {
+			if (OC.currentUser && OCA.Files.App && OC.config.session_keepalive) {
 				// start on load - we ask the server every 5 minutes
 				var func = _.bind(OCA.Files.App.fileList.updateStorageStatistics, OCA.Files.App.fileList);
 				var updateStorageStatisticsInterval = 5*60*1000;
@@ -333,7 +358,7 @@
 		 * - JS periodically checks for this cookie and then knows when the download has started to call the callback
 		 *
 		 * @param {string} url download URL
-		 * @param {function} callback function to call once the download has started
+		 * @param {Function} callback function to call once the download has started
 		 */
 		handleDownload: function(url, callback) {
 			var randomToken = Math.random().toString(36).substring(2),
@@ -357,6 +382,7 @@
 	};
 
 	Files._updateStorageStatisticsDebounced = _.debounce(Files._updateStorageStatistics, 250);
+	Files._updateStorageQuotasThrottled = _.throttle(Files._updateStorageQuotas, 30000);
 	OCA.Files.Files = Files;
 })();
 
