@@ -4,9 +4,10 @@
  *
  * @author André Gaul <gaul@web-yard.de>
  * @author Arthur Schiwon <blizzz@arthur-schiwon.de>
- * @author Bart Visscher <bartv@thisnet.nl>
  * @author Christian Berendt <berendt@b1-systems.de>
  * @author Christopher T. Johnson <ctjctj@gmail.com>
+ * @author Daniel Kesselberg <mail@danielkesselberg.de>
+ * @author enoch <lanxenet@hotmail.com>
  * @author Johan Björk <johanimon@gmail.com>
  * @author Jörn Friedrich Dreyer <jfd@butonic.de>
  * @author Martin Mattel <martin.mattel@diemattels.at>
@@ -15,6 +16,7 @@
  * @author Philipp Kapfer <philipp.kapfer@gmx.at>
  * @author Robin Appelman <robin@icewind.nl>
  * @author Robin McCorkell <robin@mccorkell.me.uk>
+ * @author Roeland Jago Douma <roeland@famdouma.nl>
  * @author Thomas Müller <thomas.mueller@tmit.eu>
  * @author Vincent Petry <pvince81@owncloud.com>
  *
@@ -30,18 +32,19 @@
  * GNU Affero General Public License for more details.
  *
  * You should have received a copy of the GNU Affero General Public License, version 3,
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program. If not, see <http://www.gnu.org/licenses/>
  *
  */
 
 namespace OCA\Files_External\Lib\Storage;
 
 use Aws\Result;
-use Aws\S3\S3Client;
 use Aws\S3\Exception\S3Exception;
+use Aws\S3\S3Client;
 use Icewind\Streams\CallbackWrapper;
 use Icewind\Streams\IteratorDirectory;
 use OC\Cache\CappedMemoryCache;
+use OC\Files\Cache\CacheEntry;
 use OC\Files\ObjectStore\S3ConnectionTrait;
 use OC\Files\ObjectStore\S3ObjectTrait;
 use OCP\Constants;
@@ -54,18 +57,21 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 		return false;
 	}
 
-	/**
-	 * @var int in seconds
-	 */
-	private $rescanDelay = 10;
-
 	/** @var CappedMemoryCache|Result[] */
 	private $objectCache;
 
+	/** @var CappedMemoryCache|bool[] */
+	private $directoryCache;
+
+	/** @var CappedMemoryCache|array */
+	private $filesCache;
+
 	public function __construct($parameters) {
 		parent::__construct($parameters);
 		$this->parseParams($parameters);
 		$this->objectCache = new CappedMemoryCache();
+		$this->directoryCache = new CappedMemoryCache();
+		$this->filesCache = new CappedMemoryCache();
 	}
 
 	/**
@@ -95,6 +101,8 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 
 	private function clearCache() {
 		$this->objectCache = new CappedMemoryCache();
+		$this->directoryCache = new CappedMemoryCache();
+		$this->filesCache = new CappedMemoryCache();
 	}
 
 	private function invalidateCache($key) {
@@ -106,6 +114,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 				unset($this->objectCache[$existingKey]);
 			}
 		}
+		unset($this->directoryCache[$key], $this->filesCache[$key]);
 	}
 
 	/**
@@ -130,6 +139,42 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 		return $this->objectCache[$key];
 	}
 
+	/**
+	 * Return true if directory exists
+	 *
+	 * There are no folders in s3. A folder like structure could be archived
+	 * by prefixing files with the folder name.
+	 *
+	 * Implementation from flysystem-aws-s3-v3:
+	 * https://github.com/thephpleague/flysystem-aws-s3-v3/blob/8241e9cc5b28f981e0d24cdaf9867f14c7498ae4/src/AwsS3Adapter.php#L670-L694
+	 *
+	 * @param $path
+	 * @return bool
+	 * @throws \Exception
+	 */
+	private function doesDirectoryExist($path) {
+		if (!isset($this->directoryCache[$path])) {
+			// Maybe this isn't an actual key, but a prefix.
+			// Do a prefix listing of objects to determine.
+			try {
+				$result = $this->getConnection()->listObjects([
+					'Bucket' => $this->bucket,
+					'Prefix' => rtrim($path, '/'),
+					'MaxKeys' => 1,
+					'Delimiter' => '/',
+				]);
+				$this->directoryCache[$path] = ($result['Contents'][0]['Key'] === rtrim($path, '/') . '/') || $result['CommonPrefixes'];
+			} catch (S3Exception $e) {
+				if ($e->getStatusCode() === 403) {
+					$this->directoryCache[$path] = false;
+				}
+				throw $e;
+			}
+		}
+
+		return $this->directoryCache[$path];
+	}
+
 	/**
 	 * Updates old storage ids (v0.2.1 and older) that are based on key and secret to new ones based on the bucket name.
 	 * TODO Do this in an update.php. requires iterating over all users and loading the mount.json from their home
@@ -195,7 +240,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 			));
 			$this->testTimeout();
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 
@@ -233,7 +278,6 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 		} catch (\Exception $e) {
 			return $this->batchDelete();
 		}
-		return false;
 	}
 
 	private function batchDelete($path = null) {
@@ -263,7 +307,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 				// we reached the end when the list is no longer truncated
 			} while ($objects['IsTruncated']);
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 		return true;
@@ -290,24 +334,34 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 				// sub folders
 				if (is_array($result['CommonPrefixes'])) {
 					foreach ($result['CommonPrefixes'] as $prefix) {
-						$files[] = substr(trim($prefix['Prefix'], '/'), strlen($path));
+						$directoryName = trim($prefix['Prefix'], '/');
+						$files[] = substr($directoryName, strlen($path));
+						$this->directoryCache[$directoryName] = true;
 					}
 				}
-				foreach ($result['Contents'] as $object) {
-					if (isset($object['Key']) && $object['Key'] === $path) {
-						// it's the directory itself, skip
-						continue;
+				if (is_array($result['Contents'])) {
+					foreach ($result['Contents'] as $object) {
+						if (isset($object['Key']) && $object['Key'] === $path) {
+							// it's the directory itself, skip
+							continue;
+						}
+						$file = basename(
+							isset($object['Key']) ? $object['Key'] : $object['Prefix']
+						);
+						$files[] = $file;
+
+						// store this information for later usage
+						$this->filesCache[$path . $file] = [
+							'ContentLength' => $object['Size'],
+							'LastModified' => (string)$object['LastModified'],
+						];
 					}
-					$file = basename(
-						isset($object['Key']) ? $object['Key'] : $object['Prefix']
-					);
-					$files[] = $file;
 				}
 			}
 
 			return IteratorDirectory::wrap($files);
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 	}
@@ -316,36 +370,84 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 		$path = $this->normalizePath($path);
 
 		try {
-			$stat = array();
+			$stat = [];
 			if ($this->is_dir($path)) {
 				//folders don't really exist
 				$stat['size'] = -1; //unknown
-				$stat['mtime'] = time() - $this->rescanDelay * 1000;
-			} else {
-				$result = $this->headObject($path);
-
-				$stat['size'] = $result['ContentLength'] ? $result['ContentLength'] : 0;
-				if (isset($result['Metadata']['lastmodified'])) {
-					$stat['mtime'] = strtotime($result['Metadata']['lastmodified']);
-				} else {
-					$stat['mtime'] = strtotime($result['LastModified']);
+				$stat['mtime'] = time();
+				$cacheEntry = $this->getCache()->get($path);
+				if ($cacheEntry instanceof CacheEntry && $this->getMountOption('filesystem_check_changes', 1) !== 1) {
+					$stat['size'] = $cacheEntry->getSize();
+					$stat['mtime'] = $cacheEntry->getMTime();
 				}
+			} else {
+				$stat['size'] = $this->getContentLength($path);
+				$stat['mtime'] = strtotime($this->getLastModified($path));
 			}
 			$stat['atime'] = time();
 
 			return $stat;
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 	}
 
+	/**
+	 * Return content length for object
+	 *
+	 * When the information is already present (e.g. opendir has been called before)
+	 * this value is return. Otherwise a headObject is emitted.
+	 *
+	 * @param $path
+	 * @return int|mixed
+	 */
+	private function getContentLength($path) {
+		if (isset($this->filesCache[$path])) {
+			return (int)$this->filesCache[$path]['ContentLength'];
+		}
+
+		$result = $this->headObject($path);
+		if (isset($result['ContentLength'])) {
+			return (int)$result['ContentLength'];
+		}
+
+		return 0;
+	}
+
+	/**
+	 * Return last modified for object
+	 *
+	 * When the information is already present (e.g. opendir has been called before)
+	 * this value is return. Otherwise a headObject is emitted.
+	 *
+	 * @param $path
+	 * @return mixed|string
+	 */
+	private function getLastModified($path) {
+		if (isset($this->filesCache[$path])) {
+			return $this->filesCache[$path]['LastModified'];
+		}
+
+		$result = $this->headObject($path);
+		if (isset($result['LastModified'])) {
+			return $result['LastModified'];
+		}
+
+		return 'now';
+	}
+
 	public function is_dir($path) {
 		$path = $this->normalizePath($path);
+
+		if (isset($this->filesCache[$path])) {
+			return false;
+		}
+
 		try {
-			return $this->isRoot($path) || $this->headObject($path . '/');
+			return $this->isRoot($path) || $this->doesDirectoryExist($path);
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 	}
@@ -358,14 +460,14 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 		}
 
 		try {
-			if ($this->headObject($path)) {
+			if (isset($this->filesCache[$path]) || $this->headObject($path)) {
 				return 'file';
 			}
-			if ($this->headObject($path . '/')) {
+			if ($this->doesDirectoryExist($path)) {
 				return 'dir';
 			}
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 
@@ -391,7 +493,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 			$this->deleteObject($path);
 			$this->invalidateCache($path);
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 
@@ -404,15 +506,21 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 		switch ($mode) {
 			case 'r':
 			case 'rb':
+				// Don't try to fetch empty files
+				$stat = $this->stat($path);
+				if (is_array($stat) && isset($stat['size']) && $stat['size'] === 0) {
+					return fopen('php://memory', $mode);
+				}
+
 				try {
 					return $this->readObject($path);
 				} catch (S3Exception $e) {
-					\OCP\Util::logException('files_external', $e);
+					\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 					return false;
 				}
 			case 'w':
 			case 'wb':
-				$tmpFile = \OCP\Files::tmpFile();
+				$tmpFile = \OC::$server->getTempManager()->getTemporaryFile();
 
 				$handle = fopen($tmpFile, 'w');
 				return CallbackWrapper::wrap($handle, null, null, function () use ($path, $tmpFile) {
@@ -433,7 +541,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 				} else {
 					$ext = '';
 				}
-				$tmpFile = \OCP\Files::tmpFile($ext);
+				$tmpFile = \OC::$server->getTempManager()->getTemporaryFile($ext);
 				if ($this->file_exists($path)) {
 					$source = $this->readObject($path);
 					file_put_contents($tmpFile, $source);
@@ -485,7 +593,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 				$this->testTimeout();
 			}
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 
@@ -506,7 +614,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 				));
 				$this->testTimeout();
 			} catch (S3Exception $e) {
-				\OCP\Util::logException('files_external', $e);
+				\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 				return false;
 			}
 		} else {
@@ -520,7 +628,7 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 				));
 				$this->testTimeout();
 			} catch (S3Exception $e) {
-				\OCP\Util::logException('files_external', $e);
+				\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 				return false;
 			}
 
@@ -573,13 +681,10 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 	}
 
 	public function test() {
-		$test = $this->getConnection()->getBucketAcl(array(
-			'Bucket' => $this->bucket,
-		));
-		if (isset($test) && !is_null($test->getPath('Owner/ID'))) {
-			return true;
-		}
-		return false;
+		$this->getConnection()->headBucket([
+			'Bucket' => $this->bucket
+		]);
+		return true;
 	}
 
 	public function getId() {
@@ -591,12 +696,11 @@ class AmazonS3 extends \OC\Files\Storage\Common {
 			$source = fopen($tmpFile, 'r');
 			$this->writeObject($path, $source);
 			$this->invalidateCache($path);
-			fclose($source);
 
 			unlink($tmpFile);
 			return true;
 		} catch (S3Exception $e) {
-			\OCP\Util::logException('files_external', $e);
+			\OC::$server->getLogger()->logException($e, ['app' => 'files_external']);
 			return false;
 		}
 	}
