@@ -292,7 +292,7 @@ class ShareAPIController extends OCSController {
 		$result = \OC::$server->getContactsManager()->search($query, [$property]);
 		foreach ($result as $r) {
 			foreach ($r[$property] as $value) {
-				if ($value === $query) {
+				if ($value === $query && $r['FN']) {
 					return $r['FN'];
 				}
 			}
@@ -490,16 +490,21 @@ class ShareAPIController extends OCSController {
 					throw new OCSNotFoundException($this->l->t('Public upload is only possible for publicly shared folders'));
 				}
 
-				$share->setPermissions(
-					Constants::PERMISSION_READ |
+				$permissions = Constants::PERMISSION_READ |
 					Constants::PERMISSION_CREATE |
 					Constants::PERMISSION_UPDATE |
-					Constants::PERMISSION_DELETE
-				);
+					Constants::PERMISSION_DELETE;
 			} else {
-				$share->setPermissions(Constants::PERMISSION_READ);
+				$permissions = Constants::PERMISSION_READ;
 			}
 
+			// TODO: It might make sense to have a dedicated setting to allow/deny converting link shares into federated ones
+			if (($permissions & Constants::PERMISSION_READ) && $this->shareManager->outgoingServer2ServerSharesAllowed()) {
+				$permissions |= Constants::PERMISSION_SHARE;
+			}
+
+			$share->setPermissions($permissions);
+
 			// Set password
 			if ($password !== '') {
 				$share->setPassword($password);
@@ -601,12 +606,12 @@ class ShareAPIController extends OCSController {
 
 		$shares = array_merge($userShares, $groupShares, $circleShares, $roomShares);
 
-		$shares = array_filter($shares, function(IShare $share) {
+		$filteredShares = array_filter($shares, function(IShare $share) {
 			return $share->getShareOwner() !== $this->currentUser;
 		});
 
 		$formatted = [];
-		foreach ($shares as $share) {
+		foreach ($filteredShares as $share) {
 			if ($this->canAccessShare($share)) {
 				try {
 					$formatted[] = $this->formatShare($share);
@@ -651,7 +656,8 @@ class ShareAPIController extends OCSController {
 		$resharingRight = false;
 		$known = [];
 		foreach ($shares as $share) {
-			if (in_array($share->getId(), $known) || $share->getSharedWith() === $this->currentUser) {
+			if (in_array($share->getId(), $known)
+				|| ($share->getSharedWith() === $this->currentUser && $share->getShareType() === IShare::TYPE_USER)) {
 				continue;
 			}
 
@@ -840,8 +846,12 @@ class ShareAPIController extends OCSController {
 			throw new OCSNotFoundException($this->l->t('Could not lock path'));
 		}
 
-		// current User has resharing rights ?
-		$this->confirmSharingRights($node);
+		if (!($node->getPermissions() & Constants::PERMISSION_SHARE)) {
+			throw new SharingRightsException('no sharing rights on this item');
+		}
+
+		// The current top parent we have access to
+		$parent = $node;
 
 		// initiate real owner.
 		$owner = $node->getOwner()
@@ -869,10 +879,25 @@ class ShareAPIController extends OCSController {
 			$nodes[] = $node;
 		}
 
+		// The user that is requesting this list
+		$currentUserFolder = $this->rootFolder->getUserFolder($this->currentUser);
+
 		// for each nodes, retrieve shares.
 		$shares = [];
+
 		foreach ($nodes as $node) {
 			$getShares = $this->getFormattedShares($owner, $node, false, true);
+
+			$currentUserNodes = $currentUserFolder->getById($node->getId());
+			if (!empty($currentUserNodes)) {
+				$parent = array_pop($currentUserNodes);
+			}
+
+			$subPath = $currentUserFolder->getRelativePath($parent->getPath());
+			foreach ($getShares as &$share) {
+				$share['via_fileid'] = $parent->getId();
+				$share['via_path'] = $subPath;
+			}
 			$this->mergeFormattedShares($shares, $getShares);
 		}
 
@@ -1011,6 +1036,11 @@ class ShareAPIController extends OCSController {
 			}
 
 			if ($newPermissions !== null) {
+				// TODO: It might make sense to have a dedicated setting to allow/deny converting link shares into federated ones
+				if (($newPermissions & Constants::PERMISSION_READ) && $this->shareManager->outgoingServer2ServerSharesAllowed()) {
+					$newPermissions |= Constants::PERMISSION_SHARE;
+				}
+
 				$share->setPermissions($newPermissions);
 				$permissions = $newPermissions;
 			}
