@@ -22,6 +22,8 @@
 namespace OCA\Notifications;
 
 
+use GuzzleHttp\Exception\ClientException;
+use GuzzleHttp\Exception\ServerException;
 use OC\Authentication\Exceptions\InvalidTokenException;
 use OC\Authentication\Token\IProvider;
 use OC\Security\IdentityProof\Key;
@@ -66,7 +68,11 @@ class Push {
 		$this->log = $log;
 	}
 
-	public function pushToDevice(int $id, INotification $notification) {
+	public function pushToDevice(int $id, INotification $notification): void {
+		if (!$this->config->getSystemValueBool('has_internet_connection', true)) {
+			return;
+		}
+
 		$user = $this->userManager->get($notification->getUser());
 		if (!($user instanceof IUser)) {
 			return;
@@ -92,7 +98,7 @@ class Push {
 		$userKey = $this->keyManager->getKey($user);
 
 		$isTalkNotification = \in_array($notification->getApp(), ['spreed', 'talk'], true);
-		$talkApps = array_filter($devices, function($device) {
+		$talkApps = array_filter($devices, function ($device) {
 			return $device['apptype'] === 'talk';
 		});
 		$hasTalkApps = !empty($talkApps);
@@ -131,6 +137,48 @@ class Push {
 			}
 		}
 
+		$this->sendNotificationsToProxies($pushNotifications);
+	}
+
+	public function pushDeleteToDevice(string $userId, int $notificationId): void {
+		if (!$this->config->getSystemValueBool('has_internet_connection', true)) {
+			return;
+		}
+
+		$user = $this->userManager->get($userId);
+		if (!($user instanceof IUser)) {
+			return;
+		}
+
+		$devices = $this->getDevicesForUser($userId);
+		if (empty($devices)) {
+			return;
+		}
+
+		$userKey = $this->keyManager->getKey($user);
+		$pushNotifications = [];
+		foreach ($devices as $device) {
+			try {
+				$payload = json_encode($this->encryptAndSignDelete($userKey, $device, $notificationId));
+
+				$proxyServer = rtrim($device['proxyserver'], '/');
+				if (!isset($pushNotifications[$proxyServer])) {
+					$pushNotifications[$proxyServer] = [];
+				}
+				$pushNotifications[$proxyServer][] = $payload;
+			} catch (InvalidTokenException $e) {
+				// Token does not exist anymore, should drop the push device entry
+				$this->deletePushToken($device['token']);
+			} catch (\InvalidArgumentException $e) {
+				// Failed to encrypt message for device: public key is invalid
+				$this->deletePushToken($device['token']);
+			}
+		}
+
+		$this->sendNotificationsToProxies($pushNotifications);
+	}
+
+	protected function sendNotificationsToProxies(array $pushNotifications): void {
 		if (empty($pushNotifications)) {
 			return;
 		}
@@ -143,26 +191,52 @@ class Push {
 						'notifications' => $notifications,
 					],
 				]);
+			} catch (ClientException $e) {
+				// Server responded with 4xx (400 Bad Request mostlikely)
+				$response = $e->getResponse();
+			} catch (ServerException $e) {
+				// Server responded with 5xx
+				$response = $e->getResponse();
+				$body = $response->getBody();
+				$error = \is_string($body) ? $body : ('no reason given (' . $response->getStatusCode() . ')');
+
+				$this->log->debug('Could not send notification to push server [{url}]: {error}', [
+					'error' => $error,
+					'url' => $proxyServer,
+					'app' => 'notifications',
+				]);
+
+				continue;
 			} catch (\Exception $e) {
 				$this->log->logException($e, [
 					'app' => 'notifications',
-					'level' => $e->getCode() === Http::STATUS_BAD_REQUEST ? ILogger::INFO : ILogger::WARN,
+					'level' => ILogger::ERROR,
 				]);
 				continue;
 			}
 
 			$status = $response->getStatusCode();
-			if ($status !== Http::STATUS_OK && $status !== Http::STATUS_SERVICE_UNAVAILABLE) {
-				$body = $response->getBody();
-				$this->log->error('Could not send notification to push server [{url}]: {error}',[
-					'error' => \is_string($body) ? $body : 'no reason given',
+			$body = $response->getBody();
+			$bodyData = json_decode($body, true);
+
+			if (is_array($bodyData) && isset($bodyData['unknown'], $bodyData['failed'])) {
+				if (is_array($bodyData['unknown'])) {
+					// Proxy returns null when the array is empty
+					foreach ($bodyData['unknown'] as $unknownDevice) {
+						$this->deletePushTokenByDeviceIdentifier($unknownDevice);
+					}
+				}
+			} elseif ($status !== Http::STATUS_OK) {
+				$error = \is_string($body) && $bodyData === null ? $body : 'no reason given';
+				$this->log->warning('Could not send notification to push server [{url}]: {error}', [
+					'error' => $error,
 					'url' => $proxyServer,
 					'app' => 'notifications',
 				]);
-			} else if ($status === Http::STATUS_SERVICE_UNAVAILABLE && $this->config->getSystemValue('debug', false)) {
-				$body = $response->getBody();
-				$this->log->debug('Could not send notification to push server [{url}]: {error}',[
-					'error' => \is_string($body) ? $body : 'no reason given',
+			} else {
+				$error = \is_string($body) && $bodyData === null ? $body : 'no reason given';
+				$this->log->info('Push notification sent but response was not parsable, using an outdated push proxy? [{url}]: {error}', [
+					'error' => $error,
 					'url' => $proxyServer,
 					'app' => 'notifications',
 				]);
@@ -192,9 +266,8 @@ class Push {
 			'id' => $notification->getObjectId(),
 		];
 
-		// Max length of encryption is 255, so we need to make sure the subject is shorter.
-		// We also substract a buffer of 10 bytes.
-		$maxDataLength = 255 - strlen(json_encode($data)) - 10;
+		// Max length of encryption is ~240, so we need to make sure the subject is shorter.
+		$maxDataLength = 200 - strlen(json_encode($data));
 		$data['subject'] = $this->shortenJsonEncodedMultibyte($notification->getParsedSubject(), $maxDataLength);
 		if ($notification->getParsedSubject() !== $data['subject']) {
 			$data['subject'] .= '…';
@@ -202,8 +275,10 @@ class Push {
 
 		if ($isTalkNotification) {
 			$priority = 'high';
+			$type = $data['type'] === 'call' ? 'voip' : 'alert';
 		} else {
 			$priority = 'normal';
+			$type = 'alert';
 		}
 
 		if (!openssl_public_encrypt(json_encode($data), $encryptedSubject, $device['devicepublickey'], OPENSSL_PKCS1_PADDING)) {
@@ -221,6 +296,7 @@ class Push {
 			'subject' => $base64EncryptedSubject,
 			'signature' => $base64Signature,
 			'priority' => $priority,
+			'type' => $type,
 		];
 	}
 
@@ -245,6 +321,48 @@ class Push {
 		return $temp;
 	}
 
+	/**
+	 * @param Key $userKey
+	 * @param array $device
+	 * @param int $id
+	 * @return array
+	 * @throws InvalidTokenException
+	 * @throws \InvalidArgumentException
+	 */
+	protected function encryptAndSignDelete(Key $userKey, array $device, int $id): array {
+		// Check if the token is still valid...
+		$this->tokenProvider->getTokenById($device['token']);
+
+		if ($id === 0) {
+			$data = [
+				'delete-all' => true,
+			];
+		} else {
+			$data = [
+				'nid' => $id,
+				'delete' => true,
+			];
+		}
+
+		if (!openssl_public_encrypt(json_encode($data), $encryptedSubject, $device['devicepublickey'], OPENSSL_PKCS1_PADDING)) {
+			$this->log->error(openssl_error_string(), ['app' => 'notifications']);
+			throw new \InvalidArgumentException('Failed to encrypt message for device');
+		}
+
+		openssl_sign($encryptedSubject, $signature, $userKey->getPrivate(), OPENSSL_ALGO_SHA512);
+		$base64EncryptedSubject = base64_encode($encryptedSubject);
+		$base64Signature = base64_encode($signature);
+
+		return [
+			'deviceIdentifier' => $device['deviceidentifier'],
+			'pushTokenHash' => $device['pushtokenhash'],
+			'subject' => $base64EncryptedSubject,
+			'signature' => $base64Signature,
+			'priority' => 'normal',
+			'type' => 'background',
+		];
+	}
+
 	/**
 	 * @param string $uid
 	 * @return array[]
@@ -273,4 +391,16 @@ class Push {
 
 		return $query->execute() !== 0;
 	}
+
+	/**
+	 * @param string $deviceIdentifier
+	 * @return bool
+	 */
+	protected function deletePushTokenByDeviceIdentifier(string $deviceIdentifier): bool {
+		$query = $this->db->getQueryBuilder();
+		$query->delete('notifications_pushtokens')
+			->where($query->expr()->eq('deviceidentifier', $query->createNamedParameter($deviceIdentifier)));
+
+		return $query->execute() !== 0;
+	}
 }
