@@ -11,8 +11,8 @@ OC.L10N.register(
     "Groups" : "Skupiny",
     "Links" : "Odkazy",
     "Mails" : "E-maily",
-    "Federated sent" : "Sdruženě odesláno",
-    "Federated received" : "Sdruženě přijato",
+    "Federated sent" : "Federovaně odesláno",
+    "Federated received" : "Federovaně přijato",
     "Talk conversations" : "Konverzace v Talk",
     "24 hours" : "24 hodin",
     "1 hour" : "1 hodina",
@@ -31,7 +31,6 @@ OC.L10N.register(
     "Memory" : "Operační paměť",
     "Server time" : "Čas na serveru",
     "Uptime" : "Doba chodu od minulého zapnutí",
-    "Time Servers" : "Časové servery",
     "Load" : "Vytížení",
     "Disk" : "Úložiště",
     "Mount" : "Připojení (mount)",
@@ -65,17 +64,9 @@ OC.L10N.register(
     "Size:" : "Velikost:",
     "External monitoring tool" : "Externí nástroj pro dohledování",
     "You can connect an external monitoring tool by using this end point:" : "Externí dohledovací nástroj je možné napojit pomocí tohoto koncového bodu:",
+    "Appending \"?format=json\" at the end of the URL gives you the result in JSON." : "Připojení \"?format=json\" na konec URL poskytne výsledek ve formátu JSON.",
+    "Time Servers" : "Časové servery",
     "Did you know?" : "Věděli jste?",
-    "Appending \"?format=json\" at the end of the URL gives you the result in JSON format!" : "Přidáním „?format=json“ (bez uvozovek) na konci URL adresy získáte výsledek ve formátu JSON!",
-    "Last 24 hours" : "Posledních 24 hodin",
-    "Last 1 hour" : "Poslední hodina",
-    "Last 5 mins" : "Uplynulých 5 minut",
-    "CPU load" : "Vytížení procesoru",
-    "Memory usage" : "Využití paměti",
-    "Storage" : "Úložiště",
-    "Users:" : "Uživatelé:",
-    "Nextcloud" : "Nextcloud",
-    "Apps installed:" : "Nainstalované aplikace:",
-    "Apps updates available:" : "Jsou k dispozici aktualizace aplikací:"
+    "Appending \"?format=json\" at the end of the URL gives you the result in JSON format!" : "Přidáním „?format=json“ (bez uvozovek) na konci URL adresy získáte výsledek ve formátu JSON!"
 },
 "nplurals=4; plural=(n == 1 && n % 1 == 0) ? 0 : (n >= 2 && n <= 4 && n % 1 == 0) ? 1: (n % 1 != 0 ) ? 2 : 3;");
