@@ -5,6 +5,7 @@ OC.L10N.register(
     "This can usually be fixed by giving the webserver write access to the config directory" : "Ovo se obično može ispraviti tako da se web poslužitelju dopusti pristup za pisanje u direktoriju config",
     "Or, if you prefer to keep config.php file read only, set the option \"config_is_read_only\" to true in it." : "Ako želite da datoteku config.php ostane samo za čitanje, postavite opciju „config_is_read_only” na „true”.",
     "See %s" : "Pogledajte %s",
+    "This can usually be fixed by giving the webserver write access to the config directory." : "Ovo se obično može ispraviti tako da se web-poslužitelju dopusti pristup za pisanje u direktoriju config.",
     "Or, if you prefer to keep config.php file read only, set the option \"config_is_read_only\" to true in it. See %s" : "Ako želite da datoteku config.php ostane samo za čitanje, postavite opciju „config_is_read_only” na „true”. Pogledajte %s",
     "The files of the app %1$s were not replaced correctly. Make sure it is a version compatible with the server." : "Datoteke aplikacije %1$s nisu ispravno zamijenjene. Provjerite je li inačica kompatibilna s poslužiteljem.",
     "Sample configuration detected" : "Pronađena ogledna konfiguracija",
@@ -21,12 +22,12 @@ OC.L10N.register(
     "PHP %s or higher is required." : "PHP inačica treba biti %s ili viša.",
     "PHP with a version lower than %s is required." : "Potreban je PHP inačice manje od %s.",
     "%sbit or higher PHP required." : "Potreban je %s-bitni ili viši PHP.",
-    "Following databases are supported: %s" : "Podržane su sljedeće baze podataka: %s",
+    "The following databases are supported: %s" : "Podržane su sljedeće baze podataka: %s",
     "The command line tool %s could not be found" : "Alat naredbenog retka %s nije pronađen",
     "The library %s is not available." : "Biblioteka %s nije dostupna.",
     "Library %1$s with a version higher than %2$s is required - available version %3$s." : "Potrebna je biblioteka %1$s inačice veće od %2$s – dostupna je inačica %3$s.",
     "Library %1$s with a version lower than %2$s is required - available version %3$s." : "Potrebna je biblioteka %1$s inačice manje od %2$s – dostupna je inačica %3$s.",
-    "Following platforms are supported: %s" : "Podržane su sljedeće platforme: %s",
+    "The following platforms are supported: %s" : "Podržane su sljedeće platforme: %s",
     "Server version %s or higher is required." : "Inačica poslužitelja treba biti %s ili viša.",
     "Server version %s or lower is required." : "Potrebna je inačica poslužitelja %s ili niža.",
     "Logged in user must be an admin or sub admin" : "Prijavljeni korisnik mora biti administrator ili podadministrator",
@@ -186,13 +187,14 @@ OC.L10N.register(
     "Cannot write into \"config\" directory" : "Pisanje u direktorij „config” nije moguće",
     "This can usually be fixed by giving the webserver write access to the config directory. See %s" : "Ovo se obično može popraviti tako da se web poslužitelju dopusti pristup za pisanje u konfiguracijski direktorij. Pogledajte %s",
     "Cannot write into \"apps\" directory" : "Nije moguće pisati u direktorij „apps”",
+    "This can usually be fixed by giving the webserver write access to the apps directory or disabling the appstore in the config file." : "Ovo se obično može popraviti tako da se web-poslužitelju dopusti pristup za pisanje u aplikacijski direktorij ili onemogućivanjem trgovine aplikacijama (App Store) u konfiguracijskoj datoteci.",
     "Cannot create \"data\" directory" : "Nije moguće stvoriti direktorij „data”",
     "This can usually be fixed by giving the webserver write access to the root directory. See %s" : "Ovo se obično može popraviti tako da se web poslužitelju dopusti pristup za pisanje u korijenski direktorij. Pogledajte %s",
     "Permissions can usually be fixed by giving the webserver write access to the root directory. See %s." : "Dopuštenja se obično mogu popraviti tako da se web poslužitelju dopusti pristup za pisanje u korijenski direktorij. Pogledajte %s.",
     "Setting locale to %s failed" : "Neuspješno postavljanje regionalne sheme na %s",
     "Please install one of these locales on your system and restart your webserver." : "Instalirajte jednu od ovih regionalnih shema u svoj sustav i ponovno pokrenite svoj web poslužitelj.",
-    "Please ask your server administrator to install the module." : "Zamolite svog administratora poslužitelja da instalira modul.",
     "PHP module %s not installed." : "PHP modul %s nije instaliran.",
+    "Please ask your server administrator to install the module." : "Zamolite svog administratora poslužitelja da instalira modul.",
     "PHP setting \"%s\" is not set to \"%s\"." : "PHP postavka „%s” nije postavljena na „%s”.",
     "Adjusting this setting in php.ini will make Nextcloud run again" : "Podešavanjem ove postavke u php.ini možete ponovno pokrenuti Nextcloud",
     "mbstring.func_overload is set to \"%s\" instead of the expected value \"0\"" : "mbstring.func_overload je postavljen na „%s” umjesto očekivane vrijednosti „0”",
@@ -205,8 +207,8 @@ OC.L10N.register(
     "Please ask your server administrator to restart the web server." : "Molimo zamolite svog administratora poslužitelja da ponovno pokrene web poslužitelj.",
     "PostgreSQL >= 9 required" : "Potreban je PostgreSQL >= 9",
     "Please upgrade your database version" : "Ažurirajte svoju inačicu baze podataka",
-    "Please change the permissions to 0770 so that the directory cannot be listed by other users." : "Promijenite dozvole na 0770 tako da se tim direktorijem ne mogu služiti drugi korisnici.",
     "Your data directory is readable by other users" : "Vaš podatkovni direktorij mogu čitati drugi korisnici",
+    "Please change the permissions to 0770 so that the directory cannot be listed by other users." : "Promijenite dozvole na 0770 tako da se tim direktorijem ne mogu služiti drugi korisnici.",
     "Your data directory must be an absolute path" : "Vaš podatkovni direktorij mora imati apsolutni put",
     "Check the value of \"datadirectory\" in your configuration" : "Provjerite vrijednost „datadirectory” u svojoj konfiguraciji",
     "Your data directory is invalid" : "Vaš je podatkovni direktorij nevažeći",
@@ -222,10 +224,8 @@ OC.L10N.register(
     "Storage connection error. %s" : "Pogreška veze pohrane. %s",
     "Storage is temporarily not available" : "Pohrana privremeno nije dostupna",
     "Storage connection timeout. %s" : "Istek veze pohrane. %s",
-    "Create" : "Stvori",
-    "Change" : "Promijeni",
-    "Delete" : "Izbriši",
-    "Share" : "Dijeli",
+    "Following databases are supported: %s" : "Podržane su sljedeće baze podataka: %s",
+    "Following platforms are supported: %s" : "Podržane su sljedeće platforme: %s",
     "Overview" : "Pregled",
     "Basic settings" : "Osnovne postavke",
     "Sharing" : "Dijeljenje",
@@ -233,31 +233,6 @@ OC.L10N.register(
     "Groupware" : "Groupware",
     "Personal info" : "Osobne informacije",
     "Mobile & desktop" : "Mobilni uređaji i osobna računala",
-    "Unlimited" : "Neograničeno",
-    "Verifying" : "Provjeravanje",
-    "Verifying …" : "Provjeravanje…",
-    "Verify" : "Provjeri",
-    "Sharing %s failed, because the backend does not allow shares from type %i" : "Dijeljenje %s nije uspjelo jer pozadina ne dopušta dijeljenja iz vrste %i",
-    "Sharing %s failed, because the file does not exist" : "Dijeljenje %s nije uspjelo jer ta datoteka ne postoji",
-    "Sharing %s failed, because you can not share with yourself" : "Dijeljenje %s nije uspjelo jer ne možete dijeliti sa sobom",
-    "Sharing %1$s failed, because the user %2$s does not exist" : "Dijeljenje %1$s nije uspjelo jer korisnik %2$s ne postoji",
-    "Sharing %1$s failed, because the user %2$s is not a member of any groups that %3$s is a member of" : "Dijeljenje %1$s nije uspjelo jer korisnik %2$s nije član niti jedne grupe u kojoj je %3$s član",
-    "Sharing %1$s failed, because this item is already shared with %2$s" : "Dijeljenje %1$s nije uspjelo jer je ova stavka već podijeljena s %2$s",
-    "Sharing %1$s failed, because this item is already shared with user %2$s" : "Dijeljenje %1$s nije uspjelo jer je ova stavka već podijeljena s korisnikom %2$s",
-    "Sharing %1$s failed, because the group %2$s does not exist" : "Dijeljenje %1$s nije uspjelo jer grupa %2$s ne postoji",
-    "Sharing %1$s failed, because %2$s is not a member of the group %3$s" : "Dijeljenje %1$s nije uspjelo jer %2$s nije član grupe %3$s",
-    "You need to provide a password to create a public link, only protected links are allowed" : "Kako biste stvorili javnu poveznicu prvo morate navesti zaporku, dopuštene su samo zaštićene poveznice",
-    "Sharing %s failed, because sharing with links is not allowed" : "Dijeljenje %s nije uspjelo jer dijeljenje putem poveznica nije dopušteno",
-    "Not allowed to create a federated share with the same user" : "Nije dopušteno stvaranje udruženog dijeljenja s istim korisnikom",
-    "Sharing %1$s failed, could not find %2$s, maybe the server is currently unreachable." : "Dijeljenje %1$s nije uspjelo, nije bilo moguće pronaći %2$s, možda je poslužitelj trenutno nedostupan.",
-    "Share type %1$s is not valid for %2$s" : "Vrsta dijeljenja %1$s ne vrijedi za %2$s",
-    "Cannot set expiration date. Shares cannot expire later than %s after they have been shared" : "Nije moguće postaviti datum isteka. Nakon što su dijeljeni, ne mogu isteći kasnije od %s",
-    "Cannot set expiration date. Expiration date is in the past" : "Nije moguće postaviti datum isteka. Datum isteka je u prošlosti",
-    "Sharing failed, because the user %s is the original sharer" : "Dijeljenje nije uspjelo jer je korisnik %s izvorni dionik",
-    "Sharing %1$s failed, because the permissions exceed permissions granted to %2$s" : "Dijeljenje %1$s nije uspjelo jer dopuštenja premašuju dopuštenja za koje %2$s ima odobrenje",
-    "Sharing %s failed, because resharing is not allowed" : "Dijeljenje %s nije uspjelo jer ponovno dijeljenje nije dopušteno",
-    "Sharing %1$s failed, because the sharing backend for %2$s could not find its source" : "Dijeljenje %1$s nije uspjelo jer pozadina za %2$s nije mogla pronaći svoj izvor",
-    "Sharing %s failed, because the file could not be found in the file cache" : "Dijeljenje %s nije uspjelo jer u predmemoriji datoteke nije nađena datoteka",
     "This can usually be fixed by giving the webserver write access to the apps directory or disabling the appstore in the config file. See %s" : "Ovo se obično može popraviti tako da se web poslužitelju dopusti pristup za pisanje u aplikacijski direktorij ili onemogućivanjem appstorea u konfiguracijskoj datoteci. Pogledajte %s"
 },
 "nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;");
