@@ -5,6 +5,7 @@
  * @author Arthur Schiwon <blizzz@arthur-schiwon.de>
  * @author Bart Visscher <bartv@thisnet.nl>
  * @author Christopher Schäpers <kondou@ts.unde.re>
+ * @author Christoph Wurst <christoph@winzerhof-wurst.at>
  * @author Florin Peter <github@florin-peter.de>
  * @author Joas Schilling <coding@schilljs.com>
  * @author Jörn Friedrich Dreyer <jfd@butonic.de>
@@ -30,7 +31,7 @@
  * GNU Affero General Public License for more details.
  *
  * You should have received a copy of the GNU Affero General Public License, version 3,
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program. If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -62,9 +63,11 @@ use OC\Cache\CappedMemoryCache;
 use OC\Files\Config\MountProviderCollection;
 use OC\Files\Mount\MountPoint;
 use OC\Files\Storage\StorageFactory;
+use OC\Lockdown\Filesystem\NullStorage;
 use OCP\Files\Config\IMountProvider;
-use OCP\Files\Mount\IMountPoint;
 use OCP\Files\NotFoundException;
+use OCP\Files\Storage\IStorageFactory;
+use OCP\ILogger;
 use OCP\IUserManager;
 
 class Filesystem {
@@ -245,11 +248,11 @@ class Filesystem {
 	/**
 	 * Returns the storage factory
 	 *
-	 * @return \OCP\Files\Storage\IStorageFactory
+	 * @return IStorageFactory
 	 */
 	public static function getLoader() {
 		if (!self::$loader) {
-			self::$loader = new StorageFactory();
+			self::$loader = \OC::$server->query(IStorageFactory::class);
 		}
 		return self::$loader;
 	}
@@ -397,39 +400,71 @@ class Filesystem {
 		if ($user === null || $user === false || $user === '') {
 			throw new \OC\User\NoUserException('Attempted to initialize mount points for null user and no user in session');
 		}
+
 		if (isset(self::$usersSetup[$user])) {
 			return;
 		}
 
+		self::$usersSetup[$user] = true;
+
 		$userManager = \OC::$server->getUserManager();
 		$userObject = $userManager->get($user);
 
 		if (is_null($userObject)) {
-			\OCP\Util::writeLog('files', ' Backends provided no user object for ' . $user, \OCP\Util::ERROR);
+			\OCP\Util::writeLog('files', ' Backends provided no user object for ' . $user, ILogger::ERROR);
+			// reset flag, this will make it possible to rethrow the exception if called again
+			unset(self::$usersSetup[$user]);
 			throw new \OC\User\NoUserException('Backends provided no user object for ' . $user);
 		}
 
-		self::$usersSetup[$user] = true;
+		$realUid = $userObject->getUID();
+		// workaround in case of different casings
+		if ($user !== $realUid) {
+			$stack = json_encode(debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 50));
+			\OCP\Util::writeLog('files', 'initMountPoints() called with wrong user casing. This could be a bug. Expected: "' . $realUid . '" got "' . $user . '". Stack: ' . $stack, ILogger::WARN);
+			$user = $realUid;
 
-		/** @var \OC\Files\Config\MountProviderCollection $mountConfigManager */
-		$mountConfigManager = \OC::$server->getMountProviderCollection();
+			// again with the correct casing
+			if (isset(self::$usersSetup[$user])) {
+				return;
+			}
 
-		// home mounts are handled seperate since we need to ensure this is mounted before we call the other mount providers
-		$homeMount = $mountConfigManager->getHomeMountForUser($userObject);
-
-		self::getMountManager()->addMount($homeMount);
-
-		\OC\Files\Filesystem::getStorage($user);
-
-		// Chance to mount for other storages
-		if ($userObject) {
-			$mounts = $mountConfigManager->getMountsForUser($userObject);
-			array_walk($mounts, array(self::$mounts, 'addMount'));
-			$mounts[] = $homeMount;
-			$mountConfigManager->registerMounts($userObject, $mounts);
+			self::$usersSetup[$user] = true;
 		}
 
-		self::listenForNewMountProviders($mountConfigManager, $userManager);
+		if (\OC::$server->getLockdownManager()->canAccessFilesystem()) {
+			/** @var \OC\Files\Config\MountProviderCollection $mountConfigManager */
+			$mountConfigManager = \OC::$server->getMountProviderCollection();
+
+			// home mounts are handled seperate since we need to ensure this is mounted before we call the other mount providers
+			$homeMount = $mountConfigManager->getHomeMountForUser($userObject);
+			self::getMountManager()->addMount($homeMount);
+
+			if ($homeMount->getStorageRootId() === -1) {
+				$homeMount->getStorage()->mkdir('');
+				$homeMount->getStorage()->getScanner()->scan('');
+			}
+
+			\OC\Files\Filesystem::getStorage($user);
+
+			// Chance to mount for other storages
+			if ($userObject) {
+				$mounts = $mountConfigManager->addMountForUser($userObject, self::getMountManager());
+				$mounts[] = $homeMount;
+				$mountConfigManager->registerMounts($userObject, $mounts);
+			}
+
+			self::listenForNewMountProviders($mountConfigManager, $userManager);
+		} else {
+			self::getMountManager()->addMount(new MountPoint(
+				new NullStorage([]),
+				'/' . $user
+			));
+			self::getMountManager()->addMount(new MountPoint(
+				new NullStorage([]),
+				'/' . $user . '/files'
+			));
+		}
 		\OC_Hook::emit('OC_Filesystem', 'post_initMountPoints', array('user' => $user));
 	}
 
@@ -597,7 +632,7 @@ class Filesystem {
 	 * check if the directory should be ignored when scanning
 	 * NOTE: the special directories . and .. would cause never ending recursion
 	 *
-	 * @param String $dir
+	 * @param string $dir
 	 * @return boolean
 	 */
 	static public function isIgnoredDir($dir) {
@@ -618,14 +653,6 @@ class Filesystem {
 		return self::$defaultInstance->rmdir($path);
 	}
 
-	static public function opendir($path) {
-		return self::$defaultInstance->opendir($path);
-	}
-
-	static public function readdir($path) {
-		return self::$defaultInstance->readdir($path);
-	}
-
 	static public function is_dir($path) {
 		return self::$defaultInstance->is_dir($path);
 	}
@@ -774,7 +801,7 @@ class Filesystem {
 	 */
 	public static function normalizePath($path, $stripTrailingSlash = true, $isAbsolutePath = false, $keepUnicode = false) {
 		if (is_null(self::$normalizedPathCache)) {
-			self::$normalizedPathCache = new CappedMemoryCache();
+			self::$normalizedPathCache = new CappedMemoryCache(2048);
 		}
 
 		/**
@@ -791,7 +818,7 @@ class Filesystem {
 			return self::$normalizedPathCache[$cacheKey];
 		}
 
-		if ($path == '') {
+		if ($path === '') {
 			return '/';
 		}
 
@@ -800,38 +827,29 @@ class Filesystem {
 			$path = \OC_Util::normalizeUnicode($path);
 		}
 
-		//no windows style slashes
-		$path = str_replace('\\', '/', $path);
+		//add leading slash, if it is already there we strip it anyway
+		$path = '/' . $path;
 
-		//add leading slash
-		if ($path[0] !== '/') {
-			$path = '/' . $path;
-		}
+		$patterns = [
+			'/\\\\/s',          // no windows style slashes
+			'/\/\.(\/\.)?\//s', // remove '/./'
+			'/\/{2,}/s',        // remove sequence of slashes
+			'/\/\.$/s',         // remove trailing /.
+		];
 
-		// remove '/./'
-		// ugly, but str_replace() can't replace them all in one go
-		// as the replacement itself is part of the search string
-		// which will only be found during the next iteration
-		while (strpos($path, '/./') !== false) {
-			$path = str_replace('/./', '/', $path);
-		}
-		// remove sequences of slashes
-		$path = preg_replace('#/{2,}#', '/', $path);
+		do {
+			$count = 0;
+			$path = preg_replace($patterns, '/', $path, -1, $count);
+		} while ($count > 0);
 
 		//remove trailing slash
-		if ($stripTrailingSlash and strlen($path) > 1 and substr($path, -1, 1) === '/') {
-			$path = substr($path, 0, -1);
+		if ($stripTrailingSlash && strlen($path) > 1) {
+			$path = rtrim($path, '/');
 		}
 
-		// remove trailing '/.'
-		if (substr($path, -2) == '/.') {
-			$path = substr($path, 0, -2);
-		}
+		self::$normalizedPathCache[$cacheKey] = $path;
 
-		$normalizedPath = $path;
-		self::$normalizedPathCache[$cacheKey] = $normalizedPath;
-
-		return $normalizedPath;
+		return $path;
 	}
 
 	/**
