@@ -1,8 +1,12 @@
 <?php
+
+declare(strict_types=1);
+
 /**
  * @copyright Copyright (c) 2016, ownCloud, Inc.
  *
  * @author Andreas Fischer <bantu@owncloud.com>
+ * @author Daniel Kesselberg <mail@danielkesselberg.de>
  * @author Hendrik Leppelsack <hendrik@leppelsack.de>
  * @author Jens-Christian Fischer <jens-christian.fischer@switch.ch>
  * @author Joas Schilling <coding@schilljs.com>
@@ -14,6 +18,7 @@
  * @author Roeland Jago Douma <roeland@famdouma.nl>
  * @author Thomas Tanghus <thomas@tanghus.net>
  * @author Vincent Petry <pvince81@owncloud.com>
+ * @author Xheni Myrtaj <myrtajxheni@gmail.com>
  *
  * @license AGPL-3.0
  *
@@ -27,13 +32,14 @@
  * GNU Affero General Public License for more details.
  *
  * You should have received a copy of the GNU Affero General Public License, version 3,
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program. If not, see <http://www.gnu.org/licenses/>
  *
  */
 
 namespace OC\Files\Type;
 
 use OCP\Files\IMimeTypeDetector;
+use OCP\ILogger;
 use OCP\IURLGenerator;
 
 /**
@@ -44,6 +50,10 @@ use OCP\IURLGenerator;
  * @package OC\Files\Type
  */
 class Detection implements IMimeTypeDetector {
+
+	private const CUSTOM_MIMETYPEMAPPING = 'mimetypemapping.json';
+	private const CUSTOM_MIMETYPEALIASES = 'mimetypealiases.json';
+
 	protected $mimetypes = [];
 	protected $secureMimeTypes = [];
 
@@ -54,6 +64,9 @@ class Detection implements IMimeTypeDetector {
 	/** @var IURLGenerator */
 	private $urlGenerator;
 
+	/** @var ILogger */
+	private $logger;
+
 	/** @var string */
 	private $customConfigDir;
 
@@ -62,13 +75,16 @@ class Detection implements IMimeTypeDetector {
 
 	/**
 	 * @param IURLGenerator $urlGenerator
+	 * @param ILogger $logger
 	 * @param string $customConfigDir
 	 * @param string $defaultConfigDir
 	 */
 	public function __construct(IURLGenerator $urlGenerator,
-								$customConfigDir,
-								$defaultConfigDir) {
+								ILogger $logger,
+								string $customConfigDir,
+								string $defaultConfigDir) {
 		$this->urlGenerator = $urlGenerator;
+		$this->logger = $logger;
 		$this->customConfigDir = $customConfigDir;
 		$this->defaultConfigDir = $defaultConfigDir;
 	}
@@ -84,9 +100,9 @@ class Detection implements IMimeTypeDetector {
 	 * @param string $mimetype
 	 * @param string|null $secureMimeType
 	 */
-	public function registerType($extension,
-								 $mimetype,
-								 $secureMimeType = null) {
+	public function registerType(string $extension,
+								 string $mimetype,
+								 ?string $secureMimeType = null): void {
 		$this->mimetypes[$extension] = array($mimetype, $secureMimeType);
 		$this->secureMimeTypes[$mimetype] = $secureMimeType ?: $mimetype;
 	}
@@ -100,40 +116,54 @@ class Detection implements IMimeTypeDetector {
 	 *
 	 * @param array $types
 	 */
-	public function registerTypeArray($types) {
+	public function registerTypeArray(array $types): void {
 		$this->mimetypes = array_merge($this->mimetypes, $types);
 
 		// Update the alternative mimetypes to avoid having to look them up each time.
-		foreach ($this->mimetypes as $mimeType) {
-			$this->secureMimeTypes[$mimeType[0]] = isset($mimeType[1]) ? $mimeType[1]: $mimeType[0];
+		foreach ($this->mimetypes as $extension => $mimeType) {
+			if (strpos($extension, '_comment') === 0) {
+				continue;
+			}
+			$this->secureMimeTypes[$mimeType[0]] = $mimeType[1] ?? $mimeType[0];
+			if (isset($mimeType[1])) {
+				$this->secureMimeTypes[$mimeType[1]] = $mimeType[1];
+			}
 		}
 	}
 
+	private function loadCustomDefinitions(string $fileName, array $definitions): array {
+		if (file_exists($this->customConfigDir . '/' . $fileName)) {
+			$custom = json_decode(file_get_contents($this->customConfigDir . '/' . $fileName), true);
+			if (json_last_error() === JSON_ERROR_NONE) {
+				$definitions = array_merge($definitions, $custom);
+			} else {
+				$this->logger->warning('Failed to parse ' . $fileName . ': ' . json_last_error_msg());
+			}
+		}
+		return $definitions;
+	}
+
 	/**
 	 * Add the mimetype aliases if they are not yet present
 	 */
-	private function loadAliases() {
+	private function loadAliases(): void {
 		if (!empty($this->mimeTypeAlias)) {
 			return;
 		}
 
 		$this->mimeTypeAlias = json_decode(file_get_contents($this->defaultConfigDir . '/mimetypealiases.dist.json'), true);
-
-		if (file_exists($this->customConfigDir . '/mimetypealiases.json')) {
-			$custom = json_decode(file_get_contents($this->customConfigDir . '/mimetypealiases.json'), true);
-			$this->mimeTypeAlias = array_merge($this->mimeTypeAlias, $custom);
-		}
+		$this->mimeTypeAlias = $this->loadCustomDefinitions(self::CUSTOM_MIMETYPEALIASES, $this->mimeTypeAlias);
 	}
 
 	/**
 	 * @return string[]
 	 */
-	public function getAllAliases() {
+	public function getAllAliases(): array {
 		$this->loadAliases();
 		return $this->mimeTypeAlias;
 	}
 
-	public function getOnlyDefaultAliases() {
+	public function getOnlyDefaultAliases(): array {
 		$this->loadMappings();
 		$this->mimeTypeAlias = json_decode(file_get_contents($this->defaultConfigDir . '/mimetypealiases.dist.json'), true);
 		return $this->mimeTypeAlias;
@@ -142,18 +172,13 @@ class Detection implements IMimeTypeDetector {
 	/**
 	 * Add mimetype mappings if they are not yet present
 	 */
-	private function loadMappings() {
+	private function loadMappings(): void {
 		if (!empty($this->mimetypes)) {
 			return;
 		}
 
 		$mimetypeMapping = json_decode(file_get_contents($this->defaultConfigDir . '/mimetypemapping.dist.json'), true);
-
-		//Check if need to load custom mappings
-		if (file_exists($this->customConfigDir . '/mimetypemapping.json')) {
-			$custom = json_decode(file_get_contents($this->customConfigDir . '/mimetypemapping.json'), true);
-			$mimetypeMapping = array_merge($mimetypeMapping, $custom);
-		}
+		$mimetypeMapping = $this->loadCustomDefinitions(self::CUSTOM_MIMETYPEMAPPING, $mimetypeMapping);
 
 		$this->registerTypeArray($mimetypeMapping);
 	}
@@ -161,7 +186,7 @@ class Detection implements IMimeTypeDetector {
 	/**
 	 * @return array
 	 */
-	public function getAllMappings() {
+	public function getAllMappings(): array {
 		$this->loadMappings();
 		return $this->mimetypes;
 	}
@@ -172,7 +197,7 @@ class Detection implements IMimeTypeDetector {
 	 * @param string $path
 	 * @return string
 	 */
-	public function detectPath($path) {
+	public function detectPath($path): string {
 		$this->loadMappings();
 
 		$fileName = basename($path);
@@ -191,13 +216,11 @@ class Detection implements IMimeTypeDetector {
 			if ($extension !== false) {
 				$extension = strtolower($extension);
 				$extension = substr($extension, 1); //remove leading .
+				return $this->mimetypes[$extension][0] ?? 'application/octet-stream';
 			}
-			return (isset($this->mimetypes[$extension]) && isset($this->mimetypes[$extension][0]))
-				? $this->mimetypes[$extension][0]
-				: 'application/octet-stream';
-		} else {
-			return 'application/octet-stream';
 		}
+
+		return 'application/octet-stream';
 	}
 
 	/**
@@ -211,7 +234,7 @@ class Detection implements IMimeTypeDetector {
 
 		if (@is_dir($path)) {
 			// directories are easy
-			return "httpd/unix-directory";
+			return 'httpd/unix-directory';
 		}
 
 		if (function_exists('finfo_open')
@@ -272,7 +295,7 @@ class Detection implements IMimeTypeDetector {
 	 * @param string $path
 	 * @return string
 	 */
-	public function detect($path) {
+	public function detect($path): string {
 		$mimeType = $this->detectPath($path);
 
 		if ($mimeType !== 'application/octet-stream') {
@@ -288,20 +311,20 @@ class Detection implements IMimeTypeDetector {
 	 * @param string $data
 	 * @return string
 	 */
-	public function detectString($data) {
-		if (function_exists('finfo_open') and function_exists('finfo_file')) {
+	public function detectString($data): string {
+		if (function_exists('finfo_open') && function_exists('finfo_file')) {
 			$finfo = finfo_open(FILEINFO_MIME);
 			$info = finfo_buffer($finfo, $data);
 			return strpos($info, ';') !== false ? substr($info, 0, strpos($info, ';')) : $info;
-		} else {
-			$tmpFile = \OC::$server->getTempManager()->getTemporaryFile();
-			$fh = fopen($tmpFile, 'wb');
-			fwrite($fh, $data, 8024);
-			fclose($fh);
-			$mime = $this->detect($tmpFile);
-			unset($tmpFile);
-			return $mime;
 		}
+
+		$tmpFile = \OC::$server->getTempManager()->getTemporaryFile();
+		$fh = fopen($tmpFile, 'wb');
+		fwrite($fh, $data, 8024);
+		fclose($fh);
+		$mime = $this->detect($tmpFile);
+		unset($tmpFile);
+		return $mime;
 	}
 
 	/**
@@ -310,12 +333,10 @@ class Detection implements IMimeTypeDetector {
 	 * @param string $mimeType
 	 * @return string
 	 */
-	public function getSecureMimeType($mimeType) {
+	public function getSecureMimeType($mimeType): string {
 		$this->loadMappings();
 
-		return isset($this->secureMimeTypes[$mimeType])
-			? $this->secureMimeTypes[$mimeType]
-			: 'application/octet-stream';
+		return $this->secureMimeTypes[$mimeType] ?? 'application/octet-stream';
 	}
 
 	/**
@@ -323,7 +344,7 @@ class Detection implements IMimeTypeDetector {
 	 * @param string $mimetype the MIME type
 	 * @return string the url
 	 */
-	public function mimeTypeIcon($mimetype) {
+	public function mimeTypeIcon($mimetype): string {
 		$this->loadAliases();
 
 		while (isset($this->mimeTypeAlias[$mimetype])) {
@@ -334,8 +355,7 @@ class Detection implements IMimeTypeDetector {
 		}
 
 		// Replace slash and backslash with a minus
-		$icon = str_replace('/', '-', $mimetype);
-		$icon = str_replace('\\', '-', $icon);
+		$icon = str_replace(['/', '\\'], '-', $mimetype);
 
 		// Is it a dir?
 		if ($mimetype === 'dir') {
